package com.emsoft.collections

import com.emsoft.common.Assert

/**
 * Utils for collections, lists, maps, arrays.
 *
 * @author Vladimir Zhukov
 */
class CollectionHelper {

    protected CollectionHelper() {
    }

    /**
     * Returns false if source isn't null and has at least one element.
     *
     * @param source
     * @return
     */
    static boolean hasElements(def source) {
        return source
        //        if (source == null)
        //            return false
        //        if (source instanceof Collection<?>)
        //            return !((Collection<?>) source).isEmpty()
        //        if (source instanceof Iterable<?>)
        //            return ((Iterable<?>) source).iterator().hasNext()
        //        if (source instanceof Map<?, ?>)
        //            return !((Map<?, ?>) source).isEmpty()
    }

    /**
     * Finds the intersection between fromSet and set and prevents it from fromSet and returns the result.
     *
     * @param fromSet
     * @param set
     * @return
     */
    private static List excludeCore(fromSet, set) {
        Assert.hasElements(set, "set")
        def list = []
        fromSet?.each { item ->
            if (!set.any { it -> it == item })
                list.add(item)
        }
        return list
    }

    static <T> List<T> exclude(Iterable<T> fromSet, Iterable<T> set) {
        return excludeCore(fromSet, set)
    }

    static <T> List<T> exclude(T[] fromSet, T... set) {
        return excludeCore(fromSet, set)
    }

    static <T> List<T> exclude(Iterable<T> fromSet, T... set) {
        return excludeCore(fromSet, set)
    }

    static <T> List<T> exclude(T[] fromSet, Iterable<T> set) {
        return excludeCore(fromSet, set)
    }

    /**
     * Returns element by index and cast to specified type. If source is null or no element at specified index (index is greater or smaller) then return null.
     *
     * @param < T >
     * @param source
     * @param index
     * @return
     */
    static <T> T getAt(Iterable<T> source, int index) {
        if (!hasElements(source) || index < 0)
            return null
        if (source instanceof List || source instanceof Map)
            return (index < source.size() ? (T)source[index] : null)
        def i = 0
        for (def iter = source.iterator(); iter.hasNext(); i++) {
            def item = iter.next()
            if (i == index)
                return item
        }
        return null
    }

    /**
     * Exclude given indexes from source collection and returns result.
     *
     * @param source
     * @param indexes
     * @return
     */
    static <T> List<T> excludeIndexes(Collection<T> source, int ... indexes) {
        Assert.hasElements(source, "source")
        Assert.hasElements(indexes, "indexes")
        def list = []
        source.eachWithIndex { item, i ->
            if (!(i in indexes))
                list << item
        }
        return list
    }

    static <T> List<T> cast(Collection<?> source, Class<T> type) {
        Assert.notNull(type, "type")
        def list = []
        if (!source)
            return list
        source.each {
            if (it)
                list << it.asType(type)
        }
        return list
        //return source?.collect { it?.asType(type) }
    }
}
