package com.emsoft.common

import com.emsoft.collections.CollectionHelper
import com.emsoft.exceptions.ExceptionHelper

/**
 * Class for checks different assertions.
 *
 * @author Vladimir Zhukov
 */
class Assert {

    protected Assert() {
    }

    /**
     * Check value on null, if value is null then throw IllegalArgumentException.
     *
     * @param value
     * @param name
     */
    static void notNull(Object value, String name) {
        if (value == null)
            ExceptionHelper.throwNullArgument(name)
    }

    /**
     * If value is null or empty then throw IllegalArgumentException.
     *
     * @param value
     * @param name
     */
    static void hasText(String value, boolean ignoreWhiteSpace = true, String name) {
        if (!StringHelper.hasText(value, ignoreWhiteSpace))
            ExceptionHelper.throwStringArgumentNullOrEmpty(name)
    }

    /**
     * If collection is null or empty then throw IllegalArgumentException.
     *
     * @param collection
     * @param name
     */
    static void hasElements(def collection, String name) {
        if (!CollectionHelper.hasElements(collection))
            ExceptionHelper.throwEmptyCollection(name)
    }

    /**
     * If collection not contains item then throw ObjectNotInCollectionException.
     *
     * @param < T >
     * @param collection
     * @param item
     * @param itemName
     */
    static <T> void contains(Collection<T> collection, T item, String itemName) {
        if (item != null && collection != null && !collection.contains(item))
            ExceptionHelper.throwObjectNotInCollection(itemName)
    }
}