package com.emsoft.common

/**
 * Common utils.
 *
 * @author Vladimir Zhukov
 */
class Helper {

    protected Helper() {
    }

    /**
     * If value is null then returns defaultValue, else returns value.
     *
     * @param < T >
     * @param value
     * @param defaultValue
     * @return
     */
    static <T> T getValue(T value, T defaultValue) {
        if (value == null)
            return defaultValue
        return value
    }

    /**
     * Returns random value, which isn't contained in values.
     *
     * @param values
     * @param r
     * @return
     */
    static int getUniqueRandomValue(int[] values, Random r) {
        int value = r.nextInt(values.length)
        if (value == 0)
            return getUniqueRandomValue(values, r)
        for (int i: values)
            if (value == i)
                return getUniqueRandomValue(values, r)
        return value
    }

    /**
     * Safe type casting.
     * If obj is not null and instance of T, then returns obj else returns null.
     *
     * @param < T >
     * @param obj
     * @param asType
     * @return
     */
    static <T> T asType(obj, Class<T> asType) {
        Assert.notNull(asType, "asType")
        if (asType.isInstance(obj))
            return obj as T
        return null
    }

    static BigDecimal stripTrailingZeros(double value, int precision = -1) {
        if (precision >= 0)
            value = value.trunc(precision)
        def dec = value.toBigDecimal()
        (value % 1) == 0 ? dec.stripTrailingZeros() : dec
    }
}