package com.emsoft.common

import com.emsoft.exceptions.ExceptionHelper

/**
 * Utils for work with properties files in UTF-8 encoding.
 *
 * @author Vladimir Zhukov
 */
class ResourceHelper {

    public static final def BUNDLE_EXTENSION = "properties"

    protected ResourceHelper() {
    }

    static PropertyResourceBundle getResourceBundleSafe(String baseName, Locale locale = Locale.default) {
        def cl = this.classLoader
        def resource = cl.getResourceAsStream("${baseName}_${locale}.${BUNDLE_EXTENSION}")
        if (resource == null)
            resource = cl.getResourceAsStream("${baseName}.${BUNDLE_EXTENSION}")
        if (resource == null)
            return null
        return new PropertyResourceBundle(new InputStreamReader(resource, "UTF8"))
    }

    /**
     * If resource not found then throw exception.
     *
     * @param baseName
     * @return
     */
    static PropertyResourceBundle getResourceBundle(String baseName, Locale locale = Locale.default) {
        def rb = getResourceBundleSafe(baseName, locale)
        if (rb == null)
            ExceptionHelper.throwResourceNotFound(baseName)
        return rb
    }

    static String getString(String baseName, String key) {
        return getResourceBundle(baseName).getString(key)
    }

    static Object getObject(String baseName, String key) {
        return getResourceBundle(baseName).getObject(key)
    }

    static String[] getStringArray(String baseName, String key) {
        return getResourceBundle(baseName).getStringArray(key)
    }
}