package com.emsoft.templates

import static com.emsoft.exceptions.ExceptionHelper.logAndThrow

/**
 * Class for wrapping methods with checked exceptions. The exception is caught, wrapped in unchecked exception
 * and throw.
 *
 * @author Vladimir Zhukov
 * @param < TInput >
 * @param < TResult >
 */
abstract class InputResultOperationTemplate<TInput, TResult> {

    /**
     * Execute method operation and returns result. If exception occurs, then it wrap in BaseRuntimeException
     * and throw.
     *
     * @param value
     * @return
     */
    TResult invoke(TInput value) {
        TResult res = null
        try {
            res = operation(value)
        }
        catch (ex) {
            logAndThrow(ex)
        }
        return res
    }

    /**
     * Method with checked exceptions, returns result and takes parameter.
     *
     * @param value
     * @return
     * @throws Exception
     */
    protected abstract TResult operation(TInput value) throws Exception
}