package com.emsoft.templates

import static com.emsoft.exceptions.ExceptionHelper.logAndThrow

/**
 * Class for wrapping methods with checked exceptions. The exception is caught, wrapped in unchecked exception
 * and throw.
 *
 * @author Vladimir Zhukov
 * @param < T >
 */
abstract class ResultOperationTemplate<T> {

    /**
     * Execute method operation and returns result. If exception occurs, then it wrap in BaseRuntimeException
     * and throw.
     *
     * @return
     */
    T invoke() {
        T res = null
        try {
            res = operation()
        }
        catch (ex) {
            logAndThrow(ex)
        }
        return res
    }

    /**
     * Method with checked exceptions, returns result.
     *
     * @return
     * @throws Exception
     */
    protected abstract T operation() throws Exception
}