package com.emsoft.templates

import static com.emsoft.exceptions.ExceptionHelper.logAndThrow

/**
 * Class for wrapping methods with checked exceptions. The exception is caught, wrapped in unchecked exception
 * and throw.
 *
 * @author Vladimir Zhukov
 */
abstract class VoidOperationTemplate {

    /**
     * Execute method operation. If exception occurs, then it wrap in BaseRuntimeException and throw.
     */
    void invoke() {
        try {
            operation()
        }
        catch (ex) {
            logAndThrow(ex)
        }
    }

    /**
     * Method with checked exceptions.
     *
     * @throws Exception
     */
    protected abstract void operation() throws Exception
}